@@ -6,6 +6,7 @@ from pathlib import Path
 from orm_collector.models import (Station,
                                   Protocol,
                                   DBData,
+                                  DBServer,
                                   DBType,
                                   DataDestiny,
                                   ServerInstance,
@@ -22,6 +23,32 @@ from sqlalchemy.sql import table
 from sqlalchemy.sql import column
 from sqlalchemy import inspect
 from rich import print
+import time
+from networktools.geo import (radius, deg2rad, ecef2llh, llh2ecef)
+
+
+def get_llh(data):
+    ecef_keys = ["x","y","z"]
+    data['ecef_x'] = float(data['ecef_x'])
+    data['ecef_y'] = float(data['ecef_y'])
+    data['ecef_z'] = float(data['ecef_z'])
+    x = data['ecef_x']
+    y = data['ecef_y']
+    z = data['ecef_z']
+    (lon, lat, h) = ecef2llh(x, y, z)
+    data["position"] = {
+        "ecef"  : {
+            "x": data['ecef_x'],
+            "y": data['ecef_y'],
+            "z": data['ecef_z']
+        },
+
+        'llh': {'lat': lat, 'lon': lon, 'z': h}
+    }
+    for k in ecef_keys:
+        key = f"ecef_{k}"
+        if key in data:
+            del data[key]
 
 def object_as_dict(obj):
     return {c.key: getattr(obj, c.key)
@@ -213,8 +240,8 @@ class SessionCollector(SH):
         position_z = kwargs.get('ECEF_Z')
         protocol_host = kwargs.get('protocol_host')
         host = kwargs.get('host')
-        port = int(kwargs.get('port'))
-        interface_port = int(kwargs.get('interface_port'))
+        port = int(kwargs.get('port',0))
+        interface_port = int(kwargs.get('interface_port',80))
         db = kwargs.get('db')
         protocol = kwargs.get('protocol')
         server = kwargs.get('server_id', "atlas")
@@ -231,15 +258,8 @@ class SessionCollector(SH):
             try:
                 point = dict(X=position_x, Y=position_y, Z=position_z)
                 # check if db and protocol exists
-                db_e = object
                 prot_e = object
                 network_id = object
-                if isinstance(db, int):
-                    db_e = self.get_dbdata_by_id(db)
-                    db_id = db_e.id
-                else:
-                    db_e = self.get_dbdata_id(db)
-                    db_id = db_e.id
                 if isinstance(protocol, int):
                     prot_e = self.get_protocol_by_id(protocol.upper())
                     prot_id = prot_e.id
@@ -259,7 +279,7 @@ class SessionCollector(SH):
                 else:
                     network_e = self.get_network_id(network)
                     network_id = network_e.id
-                if db_e and prot_e:
+                if prot_e:
                     station = Station(code=code,
                                       name=name,
                                       position_x=point['X'],
@@ -269,7 +289,6 @@ class SessionCollector(SH):
                                       protocol_host=protocol_host,
                                       port=port,
                                       host=host,
-                                      db_id=db_id,
                                       protocol_id=prot_id,
                                       active=active,
                                       server_id=server_id,
@@ -299,7 +318,6 @@ class SessionCollector(SH):
         host = kwargs.get('host')
         port = int(kwargs.get('port'))
         interface_port = int(kwargs.get('interface_port'))
-        db = kwargs.get('db')
         protocol = kwargs.get('protocol')
         server = kwargs.get('server_id', "atlas")
         active = kwargs.get('active', False)
@@ -310,16 +328,8 @@ class SessionCollector(SH):
 
         point = dict(X=position_x, Y=position_y, Z=position_z)
         # check if db and protocol exists
-        db_id = None
         prot_id = None
         network_id = None
-        if isinstance(db, int):
-            db_e = self.get_dbdata_by_id(db)
-            db_id = db_e.id
-        else:
-            db_e = self.get_dbdata_id(db)
-            db_id = db_e.id
-        kwargs["db_id"] = db_id
 
         if isinstance(protocol, int):
             prot_e = self.get_protocol_by_id(protocol.upper())
@@ -346,7 +356,7 @@ class SessionCollector(SH):
             network_id = network_e.id
         kwargs["network_id"] = server_id
 
-        if db_e and prot_e:
+        if prot_e:
             station = Station(code=code,
                               name=name,
                               position_x=point['X'],
@@ -356,7 +366,6 @@ class SessionCollector(SH):
                               protocol_host=protocol_host,
                               port=port,
                               host=host,
-                              db_id=db_id,
                               protocol_id=prot_id,
                               active=active,
                               server_id=server_id,
@@ -385,7 +394,6 @@ class SessionCollector(SH):
         new_stations = []
         if only_active:
             server = self.get_server_id(server)
-            print("Server", server)
             new_stations = self.session.query(
                 Station).filter_by(active=True, server_id=server.id)
         else:
@@ -425,14 +433,22 @@ class SessionCollector(SH):
 
     def get_station_data(self, server='atlas', active="true"):
         server = self.get_server_id(server)
-        file_q = open(self.path/"./sql/station_query.sql", "r")
-        sql_query_tpl = file_q.read()
-        sql_query = sql_query_tpl.replace("[SERVER_ID]",
-                                          str(server.id)).replace("[ACTIVE]", active)
-        file_q.close()
-        stmt = text(sql_query)
-        result = self.session.execute(stmt)
-        return result
+        file_q = Path(self.path / "sql/station_query.sql")
+        if file_q.exists():
+            sql_query_tpl = file_q.read_text()
+            sql_query = sql_query_tpl.replace("[SERVER_ID]",
+                                              str(server.id)).replace("[ACTIVE]", active)
+            stmt = text(sql_query)
+            result = self.session.execute(stmt)
+            items = []
+            for st in result:
+                item = dict(**st)
+                code = item["code"]
+                protocol = item["protocol"]
+                item["table_name"] = f"{code}_{protocol}"
+                get_llh(item)
+                items.append(item)
+            return items
 
     # Network Group
 
@@ -486,8 +502,7 @@ class SessionCollector(SH):
         else:
             return None
 
-
-    
+  
 
     # Server Instance
 
@@ -605,9 +620,8 @@ class SessionCollector(SH):
             return None
     # DBDATA TABLE
 
-    def dbdata(self, **kwargs):
+    def dbserver(self, **kwargs):
         try:
-            code = kwargs.get('code')
             path = kwargs.get('path')
             host = kwargs.get('host')
             port = int(kwargs.get('port'))
@@ -616,16 +630,17 @@ class SessionCollector(SH):
             info = kwargs.get('info')
             dbtype = kwargs.get('dbtype')
             dbname = kwargs.get('dbname')
+            hostname = kwargs.get('hostname')
             if 'type_name' in kwargs:
                 dbtype = kwargs.get('type_name')
-            instance = self.session.query(DBData).filter_by(code=code).first()
+            instance = self.session.query(DBServer).filter_by(hostname=hostname).first()
             if instance:
-                self.logger.info("DBdata existe, %s" % instance)
+                self.logger.info("DBSserver existe, %s" % instance)
                 return instance
             else:
                 db_e = self.get_dbtype_id(dbtype)
                 if db_e:
-                    instance = DBData(code=code,
+                    instance = DBServer(
                                       path=path,
                                       host=host,
                                       port=port,
@@ -633,65 +648,124 @@ class SessionCollector(SH):
                                       passw=passw,
                                       info=info,
                                       dbtype_id=db_e.id,
-                                      dbname=dbname)
-                    self.create_dbdata(instance)
-                    self.logger.info("DBdata creada, %s" % instance)
+                                      dbname=dbname, 
+                                      hostname=hostname)
+                    self.create_dbserver(instance)
+                    self.logger.info("Dbserver creada, %s" % instance)
                     return instance
                 else:
                     return None
         except Exception as ex:
-            self.logger.exception("Error al crear new dbdata, %s" % ex)
+            self.logger.exception("Error al crear new dbserver, %s" % ex)
             raise ex
 
-    def create_dbdata(self, dbdata):
-        self.session.add(dbdata)
-        self.session.commit()
+    def create_dbserver(self, dbserver, commit=True):
+        self.session.add(dbserver)
+        if commit:
+            self.session.commit()
+        else:
+            print("DBServer Data must be commited to be saved")
 
-    def update_dbdata(self, instance, new_dict):
-        t_name = 'dbdata'
-        dbdata = object_as_dict(instance)
+    def update_dbserver(self, instance, new_dict):
+        t_name = 'dbserver'
+        dbserver = object_as_dict(instance)
         for k in new_dict.keys():
-            if k in dbdata.keys():
+            if k in dbserver.keys():
                 v = new_dict[k]
-                db_e = self.get_dbtype_by_id(dbdata)
+                db_e = self.get_dbtype_by_id(dbserver)
                 new_dict['dbtype_id'] = db_e.id
-                if type(v) == type(dbdata[k]) and v != dbdata[k]:
+                if type(v) == type(dbserver[k]) and v != dbserver[k]:
                     self.update_table(t_name, instance, k, v)
 
-    def delete_dbdata(self, dbdata):
-        self.session.delete(dbdata)
+    def delete_dbserver(self, dbserver):
+        self.session.delete(dbserver)
         self.session.flush()
 
-    def get_dbdatas(self):
-        return self.get_list(DBData)
+    def get_dbservers(self):
+        return self.get_list(DBServer)
 
-    def get_dbdata_id(self, code):
+    def get_dbserver_id(self, hostname):
         # code is a unique value
-        u = self.session.query(DBData).filter_by(code=code).all()
+        u = self.session.query(DBServer).filter_by(hostname=hostname).all()
         if u:
-            return u[0]
+            return u[0].id
         else:
             return None
 
-    def get_dbdata_list_by_type(self, dtype):
+    def get_dbserver_list_by_type(self, dtype):
         # code is a unique value
-        u = self.session.query(DBData).filter_by(dbtype=dtype).all()
+        u = self.session.query(DBServer).filter_by(dbtype=dtype).all()
         return u
 
-    def get_dbdata_by_id(self, pk):
-        u = self.session.query(DBData).filter_by(id=pk).all()
+    def get_dbserver_by_id(self, pk):
+        u = self.session.query(DBServer).filter_by(id=pk).all()
         if len(u) > 0:
             return u[0]
         else:
             return None
 
+    def get_dbserver_data(self, station_id):
+        key = "[STATION_ID]"
+        file_q = Path(self.path / "sql/dbserver_query.sql")
+        if file_q.exists():
+            sql_query = file_q.read_text().replace(key,
+                                                   str(station_id))
+            stmt = text(sql_query)
+            result = self.session.execute(stmt)
+            return result
+
+    def dbdata(self, **kwargs):
+        try:
+            station = kwargs.get('station')
+            database = kwargs.get('dbserver')
+            prioridad = kwargs.get("priority")
+            
+            # buscar station, database
+            station_id = self.get_station_id(station)
+            database_id = self.get_dbserver_id(database)
+            db_e = self.get_dbdata_by_id(station_id, database_id,
+                                         prioridad)
+            if (station_id and database_id) and not db_e:
+                instance = DBData(
+                    station_id=station_id,
+                    database_id=database_id, 
+                    priority=prioridad)
+                self.create_dbdata(instance)
+                self.logger.info("Dbdata creada, %s" % instance)
+
+                return instance
+            else:
+                return db_e
+        except Exception as ex:
+            self.logger.exception("Error al crear new dbdata, %s" % ex)
+            raise ex
+
+    def create_dbdata(self, dbdata, commit=True):
+        self.session.add(dbdata)
+        if commit:
+            self.session.commit()
+        else:
+            print("DBData must be commited to be saved")
+
     def get_dbdata_data(self):
-        file_q = open(self.path/"./sql/dbdata_query.sql", "r")
-        sql_query = file_q.read()
-        file_q.close()
-        stmt = text(sql_query)
-        result = self.session.execute(stmt)
-        return result
+        file_q = Path(self.path / "sql/dbdata_query.sql")
+        if file_q.exists():
+            sql_query = file_q.read_text()
+            stmt = text(sql_query)
+            result = self.session.execute(stmt)
+            return result
+
+    def get_dbdata_by_id(self, station_id, database_id, prioridad):
+        u = self.session.query(DBData).\
+            filter_by(
+                database_id=database_id, 
+                station_id=station_id, 
+                priority=prioridad).all()
+        if len(u) > 0:
+            return u[0]
+        else:
+            return None
+
     # DBTYPE TABLE
 
     def dbtype(self, **kwargs):
